package com.usky.system.controller.web;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.exception.BusinessErrorCode;
import com.usky.system.controller.web.page.TableDataInfo;
import com.usky.system.domain.SysCheckCode;
import com.usky.system.domain.SysRole;
import com.usky.system.domain.SysUser;
import com.usky.system.domain.constants.UserConstants;
import com.usky.system.mapper.SysCheckCodeMapper;
import com.usky.system.service.*;
import com.usky.system.service.util.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 用户信息
 *
 * @author yq
 */
@RestController
@RequestMapping("/user")
public class SysUserController extends BaseController {
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ISysPostService postService;

    @Autowired
    private SysCheckCodeMapper sysCheckCodeMapper;

    Operation operation = new Operation();

    /**
     * 获取用户列表
     */
//    @Log(title = "用户管理", businessType = BusinessType.OTHER)
    @GetMapping("/list")
    public ApiResult<TableDataInfo> list(SysUser user) {
        startPage();
        List<SysUser> list = userService.selectUserList(user);
        return ApiResult.success(getDataTable(list));
    }

    /**
     * 根据用户编号获取详细信息
     */
    @GetMapping(value = {"/", "/{userId}"})
    public ApiResult<Map<String, Object>> getInfo(@PathVariable(value = "userId", required = false) Long userId) {
        Map<String, Object> ajax = new HashMap<>();
        List<SysRole> roles = roleService.selectRoleAll();
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        ajax.put("posts", postService.selectPostAll());
        if (Objects.nonNull(userId)) {
            ajax.put("data", userService.selectUserById(userId));
            ajax.put("postIds", postService.selectPostListByUserId(userId));
            ajax.put("roleIds", roleService.selectRoleListByUserId(userId));
        }
        return ApiResult.success(ajax);
    }

    /**
     * 新增用户
     */
//    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @Transactional
    @PostMapping
    public ApiResult<Void> add(@Validated @RequestBody SysUser user) {
//        SysCheckCode sysCheckCode = new SysCheckCode();
        if (UserConstants.NOT_UNIQUE.equals(userService.checkUserNameUnique(user.getUserName(), SecurityUtils.getTenantId()))) {
            return ApiResult.error(BusinessErrorCode.BIZ_BUSINESS_ERROR.getCode(), "新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
//        user.setUserType("00");
//        user.setTenantId(SecurityUtils.getTenantId());
//        user.setCreateBy(SecurityUtils.getUsername());
//
//        sysCheckCode.setPlaintext(user.getPassword());
//        try {
//            sysCheckCode.setCheckCode(operation.CBCMAC(user.getPassword().getBytes("UTF-8")));//存密码校验码
//        } catch (UnsupportedEncodingException e) {
//            e.printStackTrace();
//        }
//        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));//让校验码工具先获取原始明文密码的校验码在进行加密存储到sys_user表中
        int row = userService.insertUser(user);//将数据插入user表中后才生成user_id与表sys_check_code表关联

//        sysCheckCode.setUserId(user.getUserId());
//        sysCheckCode.setUserName(user.getUserName());
//        sysCheckCode.setDeptId(user.getDeptId());
//        sysCheckCode.setTenantId(user.getTenantId());
//        sysCheckCode.setCheckCodeType(1);
//        sysCheckCodeMapper.insert(sysCheckCode);
//
//        sysCheckCode.setUserId(user.getUserId());
//        sysCheckCode.setUserName(user.getUserName());
//        sysCheckCode.setDeptId(user.getDeptId());
//        sysCheckCode.setTenantId(user.getTenantId());
//        sysCheckCode.setPlaintext(user.getPhonenumber());
//        try {
//            sysCheckCode.setCheckCode(operation.CBCMAC(user.getPhonenumber().getBytes("UTF-8")));//存手机校验码
//            sysCheckCode.setCiphertext(operation.SM4CBCEnc(user.getPhonenumber().getBytes("UTF-8")));//存手机号密文
//        } catch (UnsupportedEncodingException e) {
//            e.printStackTrace();
//        }
//        sysCheckCode.setCheckCodeType(2);
//        sysCheckCodeMapper.insert(sysCheckCode);
        return toAjax(row);
    }

    /**
     * 新增APP用户
     */
//    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @Transactional
    @PostMapping("/appAdd")
    public ApiResult<Void> appAdd(@Validated @RequestBody SysUser user) {
        if (UserConstants.NOT_UNIQUE.equals(userService.checkUserNameUnique(user.getUserName(), user.getTenantId()))) {
            return ApiResult.error(BusinessErrorCode.BIZ_BUSINESS_ERROR.getCode(), "新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        user.setUserType("00");
        user.setTenantId(user.getTenantId());
        user.setCreateBy(user.getUserName());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        int row = userService.insertUser(user);
        return toAjax(row);
    }

    /**
     * 修改用户
     */
//    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @Transactional
    @PutMapping
    public ApiResult<Void> edit(@Validated @RequestBody SysUser user) {
        userService.checkUserAllowed(user);
        user.setUpdateBy(SecurityUtils.getUsername());
        userService.updateCheckPhoneNumber(user);
        //更新手机号校验数据
//        LambdaQueryWrapper<SysCheckCode> wrapper = new LambdaQueryWrapper<>();
//        wrapper.select(SysCheckCode::getId)
//                .eq(SysCheckCode::getUserId, user.getUserId())
//                .eq(SysCheckCode::getCheckCodeType, 2);
//        SysCheckCode sysCheckCode = sysCheckCodeMapper.selectOne(wrapper);
//        if (sysCheckCode != null) {
//            sysCheckCode.setPlaintext(user.getPhonenumber());
//            try {
//                sysCheckCode.setCiphertext(operation.SM4CBCEnc(user.getPhonenumber().getBytes("UTF-8")));
//                sysCheckCode.setCheckCode(operation.CBCMAC(user.getPhonenumber().getBytes("UTF-8")));
//                String SignatureValue = operation.HMAC(user.toString().getBytes("UTF-8"));//存入第一次计算的签名值
//                SysCheckCode sysCheckCode1 = new SysCheckCode();
//                sysCheckCode1.setUserId(user.getUserId());
//                sysCheckCode1.setDeptId(user.getDeptId());
//                sysCheckCode1.setTenantId(user.getTenantId());
//                sysCheckCode1.setUserName(user.getUserName());//被修改手机号用户名
//                sysCheckCode1.setCheckCode(SignatureValue);
//                sysCheckCode1.setPlaintext("更新手机号");
//                sysCheckCode1.setCheckCodeType(6);
//                sysCheckCodeMapper.insert(sysCheckCode1);
//                String SignatureValue1 = operation.HMAC(user.toString().getBytes("UTF-8"));
//                if (SignatureValue.equals(SignatureValue1)) {
//                    sysCheckCode.setVerificationResults(1);
//                } else {
//                    sysCheckCode.setVerificationResults(2);
//                }
//            } catch (UnsupportedEncodingException e) {
//                e.printStackTrace();
//            }
//        }
//        sysCheckCodeMapper.updateById(sysCheckCode);
        return toAjax(userService.updateUser(user));
    }

    /**
     * 删除用户
     */
//    @Log(title = "用户管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{userIds}")
    public ApiResult<Void> remove(@PathVariable Long[] userIds) {
        return toAjax(userService.deleteUserByIds(userIds));
    }

    /**
     * 删除用户
     */
//    @Log(title = "用户管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{userId}/{password}")
    public ApiResult<Void> remove(@PathVariable Long userId, @PathVariable String password) {
        String password1 = SecurityUtils.encryptPassword(password);
        return toAjax(userService.deleteUserByPwd(userId, password1));
    }

    /**
     * 重置密码
     */
    @PutMapping("/resetPwd")
    public ApiResult<Void> resetPwd(@RequestBody SysUser user) {
        userService.checkUserAllowed(user);
        userService.updateCheckPassword(user);
//        //更新密码数据
//        LambdaQueryWrapper<SysCheckCode> wrapper = new LambdaQueryWrapper<>();
//        wrapper.eq(SysCheckCode::getUserId, user.getUserId())
//                .eq(SysCheckCode::getCheckCodeType, 1);
//        SysCheckCode sysCheckCode = sysCheckCodeMapper.selectOne(wrapper);
//        if (sysCheckCode != null) {
//            try {
//                sysCheckCode.setCheckCode(operation.CBCMAC(user.getPassword().getBytes("UTF-8")));
//            } catch (UnsupportedEncodingException e) {
//                e.printStackTrace();
//            }
//        }
//        sysCheckCode.setPlaintext(user.getPassword());
//        sysCheckCodeMapper.updateById(sysCheckCode);
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        user.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(userService.resetPwd(user));
    }

    /**
     * 状态修改
     */
    @PutMapping("/changeStatus")
    public ApiResult<Void> changeStatus(@RequestBody SysUser user) {
        userService.checkUserAllowed(user);
        user.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(userService.updateUserStatus(user));
    }

    /**
     * 根据用户编号获取授权角色
     */
    @GetMapping("/authRole/{userId}")
    public ApiResult<Map<String, Object>> authRole(@PathVariable("userId") Long userId) {
        Map<String, Object> ajax = new HashMap<>();
        SysUser user = userService.selectUserById(userId);
        List<SysRole> roles = roleService.selectRolesByUserId(userId);
        ajax.put("user", user);
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        return ApiResult.success(ajax);
    }

    /**
     * 用户授权角色
     */
    @PutMapping("/authRole")
    public ApiResult<Void> insertAuthRole(Long userId, Long[] roleIds) {
        userService.insertUserAuth(userId, roleIds);
        return ApiResult.success();
    }
}
