package com.usky.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.domain.SysCheckCode;
import com.usky.system.domain.SysLogininfor;
import com.usky.system.domain.SysUser;
import com.usky.system.mapper.SysCheckCodeMapper;
import com.usky.system.mapper.SysLogininforMapper;
import com.usky.system.mapper.SysUserMapper;
import com.usky.system.service.ISysLogininforService;
import com.usky.system.service.util.Operation;
import com.usky.system.service.vo.SysLoginExportVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 系统访问日志情况信息 服务层处理
 *
 * @author yq
 */
@Service
public class SysLogininforServiceImpl extends AbstractCrudService<SysLogininforMapper, SysLogininfor> implements ISysLogininforService {

    @Autowired
    private SysLogininforMapper logininforMapper;

    @Autowired
    private SysCheckCodeMapper sysCheckCodeMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    Operation operation = new Operation();

    /**
     * 新增系统登录日志
     *
     * @param logininfor 访问日志对象
     */
    @Override
    public void insertLogininfor(SysLogininfor logininfor) {
        LambdaQueryWrapper<SysUser> wrapper = new LambdaQueryWrapper<>();
        wrapper.select(SysUser::getUserId, SysUser::getTenantId, SysUser::getDeptId)
                .eq(SysUser::getUserName, logininfor.getUserName());
        SysUser sysUser = sysUserMapper.selectOne(wrapper);
        SysCheckCode sysCheckCode = new SysCheckCode();
        Date now = new Date();
        logininfor.setAccessTime(now);
        logininfor.setTenantId(sysUser.getTenantId());
        if (sysUser.getDeptId() != null) {
            if (sysUser.getDeptId() >= Integer.MIN_VALUE && sysUser.getDeptId() <= Integer.MAX_VALUE) {
                logininfor.setDeptId(Math.toIntExact(sysUser.getDeptId()));
            } else {
                throw new BusinessException("部门ID超出Integer的最大值！");
            }
        } else {
            logininfor.setDeptId(null);
        }
        logininforMapper.insertLogininfor(logininfor);

        sysCheckCode.setPlaintext(logininfor.getInfoId().toString());
        sysCheckCode.setUserName(logininfor.getUserName());
        sysCheckCode.setTenantId(logininfor.getTenantId());
        sysCheckCode.setUserId(sysUser.getUserId());
        sysCheckCode.setDeptId(sysUser.getDeptId() != null ? sysUser.getDeptId() : null);
        sysCheckCode.setCheckCodeType(3);
        String checkCode = null;
        try {
            if (logininfor.getDeptId() == null) {
                checkCode = operation.CBCMAC((logininfor.getUserName() + logininfor.getIpaddr() + logininfor.getStatus() + logininfor.getMsg()
                        + logininfor.getAccessTime().toString() + logininfor.getTenantId().toString()).getBytes("UTF-8"));
                sysCheckCode.setCheckCode(checkCode);
                System.out.println("存入数据库校验码： " + checkCode);
            } else {
                checkCode = operation.CBCMAC((logininfor.getUserName() + logininfor.getIpaddr() + logininfor.getStatus() + logininfor.getMsg()
                        + logininfor.getAccessTime().toString() + logininfor.getDeptId().toString() + logininfor.getTenantId().toString()).getBytes("UTF-8"));
                sysCheckCode.setCheckCode(checkCode);
                System.out.println("存入数据库校验码： " + checkCode);
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        sysCheckCodeMapper.insert(sysCheckCode);
    }

    /**
     * 查询系统登录日志集合
     *
     * @param logininfor 访问日志对象
     * @return 登录记录集合
     */
    @DataScope(deptAlias = "d", userAlias = "d")
    @Override
    public List<SysLogininfor> selectLogininforList(SysLogininfor logininfor) {
        logininfor.setTenantId(SecurityUtils.getTenantId());
        return logininforMapper.selectLogininforList(logininfor);
    }

    /**
     * @param list 登录记录集合
     */
    @Override
    public void logCheck(List<SysLogininfor> list) {
        if (list.size() != 0) {
            for (int i = 0; i < list.size(); i++) {
                LambdaQueryWrapper<SysCheckCode> query = new LambdaQueryWrapper<>();
                query.select(SysCheckCode::getCheckCode)
                        .eq(SysCheckCode::getPlaintext, list.get(i).getInfoId())
                        .eq(SysCheckCode::getCheckCodeType, 3);
                SysCheckCode sysCheckCode = sysCheckCodeMapper.selectOne(query);
                if (sysCheckCode == null) {
                    throw new BusinessException("日志数据异常！");//登录日志记录有数据，而校验码表中没有数据
                }
                String checkCode1 = sysCheckCode.getCheckCode();
                //分两种情况判断：dept_id参与校验码计算与否
                try {
                    if (!Objects.nonNull(list.get(i).getDeptId())) {
                        String checkCode = operation.CBCMAC((list.get(i).getUserName() + list.get(i).getIpaddr() + list.get(i).getStatus() +
                                list.get(i).getMsg() + list.get(i).getAccessTime().toString() + list.get(i).getTenantId().toString()).getBytes("UTF-8"));
                        System.out.println("计算校验： " + checkCode);
                        System.out.println("表中数据： " + checkCode1);
                        if (!checkCode.equals(checkCode1)) {
                            throw new BusinessException("【完整性】校验失败，日志数据是被破坏或者篡改的");
                        }
                    } else {
                        String checkCode = operation.CBCMAC((list.get(i).getUserName() + list.get(i).getIpaddr() + list.get(i).getStatus() +
                                list.get(i).getMsg() + list.get(i).getAccessTime().toString() + list.get(i).getDeptId().toString() + list.get(i).getTenantId().toString()).getBytes("UTF-8"));
                        System.out.println("计算校验： " + checkCode);
                        System.out.println("表中数据： " + checkCode1);
                        if (!checkCode.equals(checkCode1)) {
                            throw new BusinessException("【完整性】校验失败，日志数据是被破坏或者篡改的");
                        }
                    }
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 查询系统登录日志集合-导出
     *
     * @param logininfor 访问日志对象
     * @return 登录记录导出集合
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "d")
    public List<SysLoginExportVO> selectLogininforListExport(SysLogininfor logininfor) {
        logininfor.setTenantId(SecurityUtils.getTenantId());
        return logininforMapper.selectLogininforListExport(logininfor);
    }

    /**
     * 批量删除系统登录日志
     *
     * @param infoIds 需要删除的登录日志ID
     * @return
     */
    @Override
    public int deleteLogininforByIds(Long[] infoIds) {
        return logininforMapper.deleteLogininforByIds(infoIds);
    }

    /**
     * 清空系统登录日志
     */
    @Override
    public void cleanLogininfor() {
        logininforMapper.cleanLogininfor();
    }
}
