package com.usky.system.service.impl;


import com.ruoyi.common.datascope.annotation.DataScope;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.domain.SysCheckCode;
import com.usky.system.domain.SysOperLog;
import com.usky.system.mapper.SysCheckCodeMapper;
import com.usky.system.mapper.SysOperLogMapper;
import com.usky.system.service.ISysOperLogService;
import com.usky.system.service.util.Operation;
import com.usky.system.service.vo.SysOperLogExportVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import sun.util.calendar.BaseCalendar;

import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 操作日志 服务层处理
 * 
 * @author yq
 */
@Service
public class SysOperLogServiceImpl extends AbstractCrudService<SysOperLogMapper, SysOperLog> implements ISysOperLogService
{
    @Autowired
    private SysOperLogMapper operLogMapper;

    @Autowired
    private SysCheckCodeMapper sysCheckCodeMapper;

    Operation operation = new Operation();
    /**
     * 新增操作日志
     * 
     * @param operLog 操作日志对象
     */
    @Override
    public void insertOperlog(SysOperLog operLog)
    {
        SysCheckCode sysCheckCode = new SysCheckCode();
        operLog.setDeptId(SecurityUtils.getLoginUser().getSysUser().getDeptId().intValue());
        operLog.setCreateBy(operLog.getOperName());

        operLog.setTenantId(SecurityUtils.getTenantId());
        Date now = new Date();
        operLog.setOperTime(now);
        operLogMapper.insertOperlog(operLog);
        sysCheckCode.setPlaintext(operLog.getOperId().toString());
        Long deptId = operLog.getDeptId() != null ? operLog.getDeptId().longValue() : null;
        sysCheckCode.setDeptId(deptId);
        sysCheckCode.setUserId(SecurityUtils.getUserId());
        sysCheckCode.setTenantId(SecurityUtils.getTenantId());
        sysCheckCode.setUserName(operLog.getOperName());
        sysCheckCode.setCheckCodeType(4);
        try {
            sysCheckCode.setCheckCode(operation.CBCMAC((operLog.getTitle()+operLog.getBusinessType().toString()+ operLog.getMethod() +operLog.getOperName()
                    +operLog.getOperUrl()+operLog.getOperIp()+operLog.getOperTime().toString()+operLog.getTenantId().toString()).getBytes("UTF-8")));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        sysCheckCodeMapper.insert(sysCheckCode);
    }

    /**
     * 查询系统操作日志集合
     * 
     * @param operLog 操作日志对象
     * @return 操作日志集合
     */
    @Override
    @DataScope(deptAlias = "d",userAlias = "d")
    public List<SysOperLog> selectOperLogList(SysOperLog operLog)
    {
        operLog.setTenantId(SecurityUtils.getTenantId());
        return operLogMapper.selectOperLogList(operLog);
    }

    /**
     * 查询系统操作日志集合-导出
     *
     * @param operLog 操作日志对象
     * @return 操作日志集合
     */
    @Override
    @DataScope(deptAlias = "d",userAlias = "d")
    public List<SysOperLogExportVO> selectOperLogListExport(SysOperLog operLog)
    {
        operLog.setTenantId(SecurityUtils.getTenantId());
        return operLogMapper.selectOperLogListExport(operLog);
    }

    /**
     * 批量删除系统操作日志
     * 
     * @param operIds 需要删除的操作日志ID
     * @return 结果
     */
    @Override
    public int deleteOperLogByIds(Long[] operIds)
    {
        return operLogMapper.deleteOperLogByIds(operIds);
    }

    /**
     * 查询操作日志详细
     * 
     * @param operId 操作ID
     * @return 操作日志对象
     */
    @Override
    public SysOperLog selectOperLogById(Long operId)
    {
        return operLogMapper.selectOperLogById(operId);
    }

    /**
     * 清空操作日志
     */
    @Override
    public void cleanOperLog()
    {
        operLogMapper.cleanOperLog();
    }
}
